#!/bin/bash
# Upgrade script
# Copyright Huawei Technologies Co., Ltd. 2023. All rights reserved.

CURRENT_DIR=$(cd $(dirname $0); pwd)
PLUGIN_NAME=porting
PLUGIN_FLAG=web

function init_path(){
    INSTALL_PATH="${ROOT_PATH}/DevKit"
    INSTALL_PLUGIN_PATH="${INSTALL_PATH}/devkitplugins"
    BACKUP_PATH="${ROOT_PATH}/portingbackup"
    PLUGIN_INSTALL_BACKEND_PATH="${INSTALL_PATH}/devkitplugins/${PLUGIN_NAME}"
    PLUGIN_INSTALL_FRONTEND_PATH="${INSTALL_PATH}/tools/nginx/html/devkit/plugins/${PLUGIN_NAME}"
    PLUGIN_BACKUP_BACKEND_PATH="${BACKUP_PATH}/backend"
    PLUGIN_BACKUP_FRONTEND_PATH="${BACKUP_PATH}/frontend"
    TOOLS_PATH="${INSTALL_PATH}/tools"
    PLUGIN_LOG_JSON_PATH="${INSTALL_PATH}/config/log/operation_log/${PLUGIN_NAME}.json"
    PLUGIN_NGINX_CONF_PATH="${INSTALL_PATH}/tools/nginx/conf/location_conf/${PLUGIN_NAME}.conf"
    ROOT_INSTALL_INI_PATH="${INSTALL_PATH}/config/install.ini"
    USER_PATH="$(grep -w USER_PATH "${ROOT_INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
    PLUGIN_INSTALL_INI_PATH="${INSTALL_PATH}/devkitplugins/${PLUGIN_NAME}/config/install.ini"
    COMPONENT_CLEAN_UP_SCRIPT="$(grep -w COMPONENT_CLEAN_UP_SCRIPT "${PLUGIN_INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
    SQLITE3_BIN_PATH=$(which sqlite3)
    SQLITE3_FRAME_FILE_PATH="${INSTALL_PATH}/devkitframework/db.sqlite3"
    SQLITE3_PLUGIN_FILE_PATH="${INSTALL_PATH}/devkitplugins/sqlite/db.sqlite3"
    PLUGIN_RUNTIME_LOG="${INSTALL_PATH}/config/log/runtime_log/${PLUGIN_NAME}"
    source ${TOOLS_PATH}/const.sh
    source ${TOOLS_PATH}/log.sh
}

function backup_data(){
    if [[ -d "${BACKUP_PATH}" ]]; then
        rm -rf "${BACKUP_PATH}"
    fi
    mkdir -p "${BACKUP_PATH}"
    mkdir -p "${PLUGIN_BACKUP_BACKEND_PATH}"
    mkdir -p "${PLUGIN_BACKUP_FRONTEND_PATH}"

    declare -A backup_array
    backup_array["${PLUGIN_INSTALL_BACKEND_PATH}"]="${PLUGIN_BACKUP_BACKEND_PATH}"
    backup_array["${PLUGIN_INSTALL_FRONTEND_PATH}"]="${PLUGIN_BACKUP_FRONTEND_PATH}"
    backup_array["${PLUGIN_LOG_JSON_PATH}"]="${BACKUP_PATH}"
    backup_array["${PLUGIN_NGINX_CONF_PATH}"]="${BACKUP_PATH}"
    backup_array["${SQLITE3_PLUGIN_FILE_PATH}"]="${BACKUP_PATH}"
    backup_array["${PLUGIN_RUNTIME_LOG}"]="${BACKUP_PATH}"

    for src_path in "${!backup_array[@]}"; do
        local dst_path=${backup_array["${src_path}"]}
        if [[ ! -e "${src_path}" ]]; then
            echo_and_log "The ${src_path} does not exist. Please check the system." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
            rm -rf "${BACKUP_PATH}"
            exit 1
        fi
        cp -rpf "${src_path}" "${dst_path}"
        if [[ $? -ne 0 ]]; then
            echo_and_log "Failed to back up ${src_path}. Please check the system." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
            rm -rf "${BACKUP_PATH}"
            exit 1
        fi 
    done
}

function upgrade_plugin(){
    # 删除原始数据，开始安装
    rm -rf "${PLUGIN_INSTALL_BACKEND_PATH}"
    rm -rf "${PLUGIN_INSTALL_FRONTEND_PATH}"
    rm -rf "${PLUGIN_LOG_JSON_PATH}"
    rm -rf "${PLUGIN_NGINX_CONF_PATH}"
    rm -rf "${SQLITE3_PLUGIN_FILE_PATH}"
    rm -rf "${PLUGIN_RUNTIME_LOG}"
    if [[ "${UPGRADE_MODE}" == "web" ]]; then
        bash "${TOOLS_PATH}/install_devkit_plugin.sh" "${PLUGIN_NAME}" "${UPGRADE_MODE}" "${UPGRADE_PACKAGE_NAME}"
        local install_flag=$?
    else
        bash "${TOOLS_PATH}/install_devkit_plugin.sh" "${PLUGIN_NAME}"
        local install_flag=$?
    fi
    if [[ "$install_flag" -ne 0 ]]; then
        echo_and_log "Upgrade failed during installation of the Kunpeng Porting Advisor of the new version." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        echo_and_log "Please check possible reasons, such as lack of disk space or available ports." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        # 数据回滚
        roll_back
        exit 1
    fi
    # 关闭服务
    gunicorn_plugin_ids=$(ps -ef | grep "devkitplugins/gunicorn_plugin.py" | grep -v 'grep' | awk '{print $2}' | xargs)
	if [[ "${gunicorn_plugin_ids}" ]]; then
		kill -9 ${gunicorn_plugin_ids}
	fi
	if [[ ${system_is_container} == 1 ]]; then
		bash ${INSTALL_PATH}/tools/docker_service.sh "stop" "gunicorn_plugin"
	else
		systemctl stop gunicorn_plugin.service
	fi
    TASKS_MIGRATIONS_BACK_PATH="${BACKUP_PATH}/backend/${PLUGIN_NAME}/tasks/migrations"
    DEPENDENCY_DICTIONARY_MIGRATIONS_BACK_PATH="${BACKUP_PATH}/backend/${PLUGIN_NAME}/dependency_dictionary/migrations"
    
    # 备份需要更新的文件
    if [[ ! -d "${PLUGIN_INSTALL_BACKEND_PATH}" ]]; then
        echo_and_log "An error occurred during installation of the new tool version because dir of the ${PLUGIN_INSTALL_BACKEND_PATH} is missing." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        roll_back
        exit 1
    fi
    NEW_BACK_PATH="${ROOT_PATH}/new_porting_bak"
    if [[ -d "${NEW_BACK_PATH}" ]]; then
        rm -rf "${NEW_BACK_PATH}"
    fi
    mkdir -p "${NEW_BACK_PATH}"
    cp -rpf "${PLUGIN_INSTALL_BACKEND_PATH}" "${NEW_BACK_PATH}"

    # 删除新安装的后端目录
    rm -rf "${PLUGIN_INSTALL_BACKEND_PATH}"
    # cp原来安装的目录到用户升级目录下
    if [[ ! -d "${PLUGIN_BACKUP_BACKEND_PATH}/${PLUGIN_NAME}" ]]; then
        echo_and_log "The backup ${PLUGIN_NAME} dir does not exist. Please check the system." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        roll_back
        exit 1
    fi
    cp -rpf "${PLUGIN_BACKUP_BACKEND_PATH}/${PLUGIN_NAME}" "${INSTALL_PLUGIN_PATH}"
    cp -rpf "${NEW_BACK_PATH}/${PLUGIN_NAME}" "${INSTALL_PLUGIN_PATH}"
    # 还原插件信息
    if [[ ! (-d "${TASKS_MIGRATIONS_BACK_PATH}" && -d "${DEPENDENCY_DICTIONARY_MIGRATIONS_BACK_PATH}" && -f "${BACKUP_PATH}/db.sqlite3") ]]; then
        echo_and_log "Backup migration dir or sqlite3 file does not exist. Please check the system." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        roll_back
        exit 1
    fi
    # 拷贝老的tasks迁移数据
    cp -rfp "${TASKS_MIGRATIONS_BACK_PATH}" "${PLUGIN_INSTALL_BACKEND_PATH}/tasks"
    if [[ $? != 0 ]]; then
        echo_and_log "Failed to restore tasks migration dir." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        roll_back
        exit 1
    fi
    # 拷贝老的dependency_dictionary迁移数据
    cp -rfp "${DEPENDENCY_DICTIONARY_MIGRATIONS_BACK_PATH}" "${PLUGIN_INSTALL_BACKEND_PATH}/dependency_dictionary"
    if [[ $? != 0 ]]; then
        echo_and_log "Failed to restore dependency_dictionary migration dir." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        roll_back
        exit 1
    fi
    # 拷贝老的db.sqlite3文件
    cp -rpf "${BACKUP_PATH}/db.sqlite3" "${INSTALL_PATH}/devkitplugins/sqlite"
    if [[ $? != 0 ]]; then
        echo_and_log "Failed to restore sqlite3 file." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
        roll_back
        exit 1
    fi

    cd "${PLUGIN_INSTALL_BACKEND_PATH}"
	manage_pyc_path=${INSTALL_PATH}/devkitplugins/manage.pyc
	export LD_LIBRARY_PATH=${USER_PATH}/libsqlite3:${USER_PATH}/libssl:${USER_PATH}/lib
    source "${INSTALL_PATH}/tools/python_venv/bin/activate"
	python3 ${manage_pyc_path} makemigrations
	make_migrations_code=$?
	python3 ${manage_pyc_path} migrate
	migrate_code=$?
    unset LD_LIBRARY_PATH
    deactivate
	if [[ "${make_migrations_code}${migrate_code}" != "00" ]]; then
        roll_back
        exit 1
	fi
	# 初始化后数据库后修改目录文件权限
	chown devkit:devkit "${PLUGIN_INSTALL_BACKEND_PATH}" -R
	chown devkit:devkit ${INSTALL_PATH}/config/log -R
	chown devkit:devkit ${INSTALL_PATH}/logs -R
    chown devkit:devkit "${SQLITE3_PLUGIN_FILE_PATH}"
    chown root:root -R ${PLUGIN_INSTALL_BACKEND_PATH}/deploy
    chown root:root ${PLUGIN_INSTALL_BACKEND_PATH}

    # 升级成功删除备份数据
    rm -rf "${NEW_BACK_PATH}"
    rm -rf "${BACKUP_PATH}"
}

function roll_back(){
    echo_and_log "Start rollback." "${DOING}" "${UPGRADE_PLUGINS_LOG}"
    rm -rf "${PLUGIN_INSTALL_BACKEND_PATH}"
    rm -rf "${PLUGIN_INSTALL_FRONTEND_PATH}"
    declare -A backed_up_array
    backed_up_array["${PLUGIN_BACKUP_BACKEND_PATH}/${PLUGIN_NAME}"]="${INSTALL_PLUGIN_PATH}"
    backed_up_array["${PLUGIN_BACKUP_FRONTEND_PATH}/${PLUGIN_NAME}"]="${INSTALL_PATH}/tools/nginx/html/devkit/plugins"
    backed_up_array["${BACKUP_PATH}/db.sqlite3"]="${SQLITE3_PLUGIN_FILE_PATH}"
    backed_up_array["${BACKUP_PATH}/${PLUGIN_NAME}"]="${INSTALL_PATH}/config/log/runtime_log/"
    backed_up_array["${BACKUP_PATH}/${PLUGIN_NAME}.json"]="${INSTALL_PATH}/config/log/operation_log/${PLUGIN_NAME}.json"
    backed_up_array["${BACKUP_PATH}/${PLUGIN_NAME}.conf"]="${INSTALL_PATH}/tools/nginx/conf/location_conf/${PLUGIN_NAME}.conf"
    for src_path in "${!backed_up_array[@]}"; do
        local dst_path=${backed_up_array["${src_path}"]}
        if [[ ! -e "${src_path}" ]]; then
            echo_and_log "Rollback failed. The backup ${src_path} does not exist." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
            update_plugin_status 6
            exit 1
        fi
        cp -rpf "${src_path}" "${dst_path}"
        if [[ $? -ne 0 ]]; then
            echo_and_log "Failed to roll back ${src_path}." "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
            update_plugin_status 6
            exit 1
        fi 
    done
    chown devkit:devkit ${PLUGIN_INSTALL_FRONTEND_PATH} -R
	chmod 400 $(find ${PLUGIN_INSTALL_FRONTEND_PATH} -type f)
	chmod 700 $(find ${PLUGIN_INSTALL_FRONTEND_PATH} -type d)
    cp -rf "${BACKUP_PATH}/${PLUGIN_NAME}.json" "${PLUGIN_LOG_JSON_PATH}"
    chown devkit:devkit ${PLUGIN_LOG_JSON_PATH}
    cp -rf "${BACKUP_PATH}/${PLUGIN_NAME}.conf" "${PLUGIN_NGINX_CONF_PATH}"
    chown devkit:devkit ${PLUGIN_NGINX_CONF_PATH}
    # 执行插件自己带的安装脚本
    plugin_script
    # 修改文件权限
    change_file_path_permission
    update_plugin_status 3
    echo_and_log "Rollback finished." "${SUCCESS}" "${UPGRADE_PLUGINS_LOG}"
    rm -rf "${NEW_BACK_PATH}"
    rm -rf "${BACKUP_PATH}"
}

function plugin_script(){
    local script_path="${CURRENT_DIR}/install.sh"
    if [[ -f "${script_path}" ]]; then
        bash "${script_path}" "${INSTALL_PATH}" "${PLUGIN_FLAG}"
        if [[ $? != 0 ]]; then
            echo_and_log "Upgrade failed!" "${ERROR}" "${UPGRADE_PLUGINS_LOG}"
            exit 1
        fi
    fi
}

function change_file_permission() {
    local file="$1"
    local num="$2"
    if [[ -f "${file}" ]]; then
        chmod "${num}" "${file}"
    fi
}

function change_file_path_permission() {
	# 修改文件的权限
	scripts=$(find ${PLUGIN_INSTALL_BACKEND_PATH} -name "*\.sh")
	if [[ "${scripts}" ]]; then
		chmod 500 ${scripts}
	fi
	chmod 700 $(find ${PLUGIN_INSTALL_BACKEND_PATH} -type d)
	chmod 710 ${PLUGIN_INSTALL_BACKEND_PATH}
	chmod 400 $(find ${INSTALL_PATH} -name "*\.pyc")
	chmod 400 $(find ${INSTALL_PATH} -name "*\.py")
	chmod 400 $(find ${INSTALL_PATH}/ -regex ".*\.css\|.*\.js\|.*\.html\|.*\.png\|.*\.svg\|.*\.gif\|.*\.PNG\|.*\.ico\|.*\.md\|.*\.hhc\|.*\.hhk\|.*\.hhp")
	if [[ -e ${INSTALL_PATH}/config/log/runtime_log/${PLUGIN_NAME}/${PLUGIN_NAME}_${PLUGIN_NAME}_cfg.json ]]; then
		chmod 660 ${INSTALL_PATH}/config/log/runtime_log/${PLUGIN_NAME}/${PLUGIN_NAME}_${PLUGIN_NAME}_cfg.json
	fi
	if [[ -e ${INSTALL_PATH}/config/log/runtime_log/plugin_config.json ]]; then
		chmod 660 ${INSTALL_PATH}/config/log/runtime_log/plugin_config.json
	fi
	if [[ -f ${PLUGIN_INSTALL_BACKEND_PATH}/${COMPONENT_CLEAN_UP_SCRIPT} ]]; then
		chmod 500 ${PLUGIN_INSTALL_BACKEND_PATH}/${COMPONENT_CLEAN_UP_SCRIPT}
	fi
	change_file_permission "${PLUGIN_INSTALL_INI_PATH}" 600
	change_file_permission "${PLUGIN_INSTALL_BACKEND_PATH}/config/install_config.json" 600
	change_file_permission "${PLUGIN_INSTALL_BACKEND_PATH}/config/version.conf" 600
	change_file_permission "${PLUGIN_INSTALL_BACKEND_PATH}/config/log/version.conf" 600
	change_file_permission "${PLUGIN_INSTALL_BACKEND_PATH}/config/log/${PLUGIN_NAME}.json" 600

	chmod 600 ${INSTALL_PATH}/config/log/operation_log/*
	chown -R devkit:devkit ${INSTALL_PLUGIN_PATH}/ssh_key
	chown root:root -R ${PLUGIN_INSTALL_BACKEND_PATH}/deploy
	chown root:root ${PLUGIN_INSTALL_BACKEND_PATH}
	chown root:root ${INSTALL_PLUGIN_PATH}
	chmod 755 ${INSTALL_PLUGIN_PATH} ${PLUGIN_INSTALL_BACKEND_PATH}/deploy ${PLUGIN_INSTALL_BACKEND_PATH}
}

# 刷新框架数据库中插件的安装状态 只需要更新升级失败场景 成功场景框架中已更新
# status={6：升级失败，回滚升级前版本失败，3：升级失败，回滚升级前版本成功}
function update_plugin_status(){
    local status=$1
    local update_time=$(date '+%Y-%m-%d %H:%M:%S')
    "$SQLITE3_BIN_PATH" "$SQLITE3_FRAME_FILE_PATH" "update pluginmanage_plugin set status='$status',progress_status=9,update_time='$update_time' where plugin_id='devkit.porting';"
}

# 判断安装系统类型 是否为docker 或者其他类型容器
function check_system_type() {
	system_is_container=0
	# 检测是否为容器，是容器返回1，否则返回0（当前工具支持docker和lxc容器）
    if [[ -f /.dockerenv ]]; then
        system_is_container=1
        return
    fi
    # docker里没有systemd
    which systemd-detect-virt >/dev/null 2>&1
    if [[ "$?" -eq 0 ]]; then
        # 检测是否为容器
        container=$(systemd-detect-virt -c)
        if [[ "$container" == "none" ]]; then
            return
        fi
        system_is_container=1
        return
    fi
}

function main(){
    # 已安装插件的路径 /opt
    ROOT_PATH=$1
    UPGRADE_MODE=$2
    UPGRADE_PACKAGE_NAME=$3
    # 初始化路径
    init_path
    create_log "${UPGRADE_PLUGINS_LOG}"
    check_system_type
    # 数据备份
    backup_data
    # 升级插件
    upgrade_plugin
}

main "$@"
